# -*- coding: utf-8 -*-
#
# HnTool rules - base class
# Copyright (C) 2010 Mauricio Vieira <mauricio@mauriciovieira.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os

class Rule:
   def __init__(self, options):
      '''Type must be "config" or "files"
      short_name
      long_name
      '''
      self.check_results = {'ok': [], 'low': [], 'medium': [], 'high': [], 'info': []}
      self.type = ""
      self.short_name = ""
      self.long_name = ""
      pass

   def requires(self):
      '''This method should return all the required files to run
      the module. Usually, it's the same as self.conf_file'''
      return None

   def analyze(self, options):
      '''Do your magic and fill self.check_results with strings'''


# -*- coding: utf-8 -*-
#
# HnTool rules - port checks
# Copyright (C) 2009-2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import re
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="ports"
        self.long_name="Checks for open ports"
        self.type="config"

    def check_ports(self, lsof_command, check_results):
        ''' Check for open ports and services '''

        out = os.popen('LC_ALL=C ' + lsof_command + ' -i -nP').read()
        services = {}

        # the regex need some improvement
        for i in re.finditer(r'([A-Za-z0-9_-]+).*:([0-9]+) \(LISTEN\)', out):
            service_name = i.group(1)
            service_port = i.group(2)
            if service_name not in services:
                services[service_name] = [service_port]
            elif service_port not in services[service_name]:
                services[service_name].append(service_port)

        if len(services) > 0:
            for service in services:
                if len(services[service]) == 1:
                    tmp_msg = 'port "' + services[service][0] + '"'
                else:
                    tmp_msg = 'ports "' + '" and "'.join(services[service]) \
                            + '"'
                check_results['info'].append('Service "' + service + '" using ' \
                                             + tmp_msg + ' found')
        else:
            check_results['ok'].append("Could not find any open door")

        return check_results

    def analyze(self, options):
        check_results = self.check_results
        lsof_bin_path = ['/bin/lsof', '/sbin/lsof', '/usr/bin/lsof', '/usr/sbin/lsof']

        # checks using lsof
        # checking if we can find the lsof command
        lsof_command = ''
        for lsof in lsof_bin_path:
            if os.path.isfile(lsof):
                lsof_command = lsof
                break

        if len(lsof_command) > 0:
            self.check_ports(lsof_command, check_results)
        else:
            check_results['info'].append('Could not find the \'lsof\' command')

        return check_results


# -*- coding: utf-8 -*-
#
# HnTool - utility functions
# Copyright (C) 2009-2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

import os
import sys
import re
import shlex

# Functions

def is_root():
    '''Method to check if hntool is running as root.'''
    if os.getuid() == 0:
        return True

def is_unix():
    '''Method to check if we have power'''
    if os.name == 'posix':
        return True
    return False

def term_len():
    return int(os.popen('stty size', 'r').read().split()[1])

def split_len(seq, length):
    result = []
    p = re.compile("(.{,"+str(length)+"})\s")
    while len(seq) > 0:
        if len(seq) < length:
            result.append(seq)
            break
        else:
            tmp,seq = (p.split(seq,1))[1:]
            result.append(tmp)
    return result

def hntool_conf_parser(pfile):
    '''This method parses a config file and returns a list with
    all the lines that aren't comments or blank'''

    result = {}

    if os.path.isfile(pfile):
        fp = open(pfile,'r') # reading the file
        for line in fp:
            # getting all the lines that aren't comments
            line = shlex.split(line, comments=True)
            if len(line) >= 2:
                result[line[0]] = line[1]
        fp.close() #closing the file

    # returns a list with all the lines
    # [['option1', 'value1'], ['option2', 'value2']]
    # and so on
    return result

def requirements_met(pfile):
    '''This method should check if all the requirements (files)
    are met (one or more files can be found on the system)'''

    for f in pfile:
        if os.path.isfile(f):
            return True

    return False


# -*- coding: utf-8 -*-
#
# hntool rules - __init__
# Copyright (C) 2009-2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

__files__ = ['authentication', 'filesystems', 'php', 'remote', 'system-wide']
__services__ = ['apache', 'proftpd', 'ports', 'postgresql', 'ssh', 'vsftpd']

__all__ = __files__ + __services__


# -*- coding: utf-8 -*-
#
# HnTool rules - php
# Copyright (C) 2009-2010 Candido Vieira <cvieira.br@gmail.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import ConfigParser
import HnTool.modules.util
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="ssh"
        self.long_name="Checks security problems on php config file"
        self.type="config"
        self.required_files = ['/etc/php5/apache2/php.ini', '/etc/php5/cli/php.ini', '/etc/php.ini']

    def requires(self):
        return self.required_files

    def analyze(self, options):
        check_results = self.check_results
        conf_files = self.required_files

        for php_conf in conf_files:
            if os.path.isfile(php_conf):

                config = ConfigParser.ConfigParser()

                try:
                    config.read(php_conf)
                except ConfigParser.ParsingError, (errno, strerror):
                    check_results['info'].append('Could not parse %s: %s' % (php_conf, strerror))
                    continue

                if not config.has_section('PHP'):
                    check_results['info'].append('%s is not a PHP config file' % (php_conf))
                    continue

                if config.has_option('PHP', 'register_globals'):
                    rg = config.get('PHP', 'register_globals').lower()
                    if rg == 'on':
                        check_results['medium'].append('Register globals is on (%s)' % (php_conf))
                    elif rg == 'off':
                        check_results['ok'].append('Register globals is off (%s)' % (php_conf))
                    else:
                        check_results['info'].append('Unknown value for register globals (%s)' % (php_conf))
                else:
                    check_results['info'].append('Register globals not found (%s)' % (php_conf))

                if config.has_option('PHP', 'safe_mode'):
                    sm = config.get('PHP', 'safe_mode').lower()
                    if sm == 'on':
                        check_results['low'].append('Safe mode is on (fake security) (%s)' % (php_conf))
                    elif sm == 'off':
                        check_results['info'].append('Safe mode is off (%s)' % (php_conf))
                    else:
                        check_results['info'].append('Unknown value for safe mode (%s)' % (php_conf))
                else:
                    check_results['info'].append('Safe mode not found (%s)' % (php_conf))

                if config.has_option('PHP', 'display_errors'):
                    de = config.get('PHP', 'display_errors').lower()
                    if de == 'on':
                        check_results['medium'].append('Display errors is on (stdout) (%s)' % (php_conf))
                    elif de == 'off':
                        check_results['ok'].append('Display errors is off (%s)' % (php_conf))
                    elif de == 'stderr':
                        check_results['info'].append('Display errors set to stderr (%s)' % (php_conf))
                    else:
                        check_results['info'].append('Unknown value for display errors (%s)' % (php_conf))
                else:
                    check_results['info'].append('Display errors not found (%s)' % (php_conf))

                if config.has_option('PHP', 'expose_php'):
                    ep = config.get('PHP', 'expose_php').lower()
                    if ep == 'on':
                        check_results['low'].append('Expose PHP is on (%s)' % (php_conf))
                    elif ep == 'off':
                        check_results['ok'].append('Expose PHP is off (%s)' % (php_conf))
                    else:
                        check_results['info'].append('Unknown value for expose PHP (%s)' % (php_conf))
                else:
                    check_results['info'].append('Expose PHP not found (%s)' % (php_conf))

        return check_results


# -*- coding: utf-8 -*-
#
# HnTool rules - protftpd
# Copyright (C) 2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import HnTool.modules.util
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
	def __init__(self, options):
		MasterRule.__init__(self, options)
		self.short_name="proftpd"
		self.long_name="Checks security problems on ProFTPd servers"
		self.type="config"

	def analyze(self, options):
		check_results = self.check_results
		proftpd_conf_file = ['/etc/proftpd.conf']
		proftpd_conf_file_found = False

		for proftpd_conf in proftpd_conf_file:
			if os.path.isfile(proftpd_conf):
				# dict with all the lines
				lines = HnTool.modules.util.hntool_conf_parser(proftpd_conf)
				proftpd_conf_file_found = True

				# Checking if ProFTPd is using the default port
				if 'Port' in lines:
					if int(lines['Port']) == 21:
						check_results['info'].append('ProFTPd is running under default port (21)')
					elif int(lines['Port']) != 21:
						check_results['info'].append('ProFTPd is running under port ' +
							                                 lines['Port'])
				else: # if we didn't found 'Ports' in lines than ProFTPd uses the default one
					check_results['info'].append('ProFTPd is running under default port (21)')

				# Checking if ProFTPd allows more than 3 login attempts
				if 'MaxLoginAttempts' in lines:
					if int(lines['MaxLoginAttempts']) > 3:
						check_results['medium'].append('ProFTPd allows more than 3 login attempts')
					elif int(lines['MaxLoginAttempts']) <= 3:
						check_results['ok'].append('ProFTPd does not allows more than 3 login attempts')
				else:
					# if we didn't found 'MaxLoginAttempts' in lines than ProFTPd uses the
					# default value for this, which is 3
					check_results['medium'].append('ProFTPd allows more than 3 login attempts')

				# Checking if ProFTPd allows root login
				if 'RootLogin' in lines:
					if lines['RootLogin'] == 'on':
						check_results['medium'].append('ProFTPd allows root login')
					elif lines['RootLogin'] == 'off':
						check_results['ok'].append('ProFTPd does not allows root login')
				else:
					# if we didn't found 'RootLogin' in lines than ProFTPd uses the
					# default value for this. By default proftpd disallows root logins
					check_results['ok'].append('ProFTPd does not allows root login')

				# Checking if ProFTPd allows footprinting
				if 'ServerIdent' in lines:
					if lines['ServerIdent'] == 'on':
						check_results['medium'].append('ProFTPd allows footprinting')
					elif lines['ServerIdent'] == 'off':
						check_results['ok'].append('ProFTPd does not allows footprinting')
				else:
					check_results['ok'].append('ProFTPd allows footprinting')

				# Checking if we chroot users into the ftp users' home directory
				if 'DefaultRoot' in lines:
					if lines['DefaultRoot'] != '~':
						check_results['medium'].append('ProFTPd does not chroot users')
					elif lines['DefaultRoot'] != '~':
						check_results['ok'].append('ProFTPd chroot users')
				else:
					check_results['medium'].append('ProFTPd does not chroot users')

		if not proftpd_conf_file_found:
			check_results['info'].append('Could not find a proftpd.conf file')

		return check_results


# -*- coding: utf-8 -*-
#
# HnTool rules - ssh
# Copyright (C) 2009-2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import HnTool.modules.util
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="ssh"
        self.long_name="Checks security problems on sshd config file"
        self.type="config"
        self.required_files = ['/etc/ssh/sshd_config', '/etc/sshd_config']

    def requires(self):
        return self.required_files

    def analyze(self, options):
        check_results = self.check_results
        ssh_conf_file = self.required_files

        for sshd_conf in ssh_conf_file:
            if os.path.isfile(sshd_conf):
                # dict with all the lines
                lines = HnTool.modules.util.hntool_conf_parser(sshd_conf)

                # Checking if SSH is using the default port
                if 'Port' in lines:
                    if int(lines['Port']) == 22:
                        check_results['low'].append('SSH is using the default port')
                    else:
                        check_results['ok'].append('SSH is not using the default port')
                else:
                    check_results['low'].append('SSH is using the default port')

                # Checking if the Root Login is allowed
                if 'PermitRootLogin' in lines:
                    if lines['PermitRootLogin'] == 'yes':
                        check_results['medium'].append('Root access allowed')
                    else:
                        check_results['ok'].append('Root access is not allowed')
                else:
                    check_results['medium'].append('Root access is allowed')

                # Checking if SSH is using protocol v2 (recommended)
                if 'Protocol' in lines:
                    if int(lines['Protocol']) == 2:
                        check_results['ok'].append('SSH is using protocol v2')
                    else:
                        check_results['high'].append('SSH is not using protocol v2')
                else:
                    check_results['high'].append('SSH is not using protocol v2')

                # Checking if empty password are allowed (shouldn't)
                if 'PermitEmptyPasswords' in lines:
                    if lines['PermitEmptyPasswords'] == 'yes':
                        check_results['high'].append('Empty passwords are allowed')
                    else:
                        check_results['ok'].append('Empty passwords are not allowed')
                else:
                    check_results['high'].append('Empty passwords are allowed')

                # Checking if X11 Forward is allowed (shouldn't)
                if 'X11Forwarding' in lines:
                    if lines['X11Forwarding'] == 'yes':
                        check_results['low'].append('X11 forward is allowed')
                    else:
                        check_results['ok'].append('X11 forward is not allowed')
                else:
                    check_results['ok'].append('X11 forward is not allowed')

                # Checking if SSH allow TCP Forward (shouldn't)
                if 'AllowTcpForwarding' in lines:
                    if lines['AllowTcpForwarding'] == 'yes':
                        check_results['low'].append('TCP forwarding is allowed')
                    else:
                        check_results['ok'].append('TCP forwarding is not allowed')
                else:
                    check_results['low'].append('TCP forwarding is allowed')

        return check_results


# -*- coding: utf-8 -*-
#
# HnTool rules - apache
# Copyright (C) 2009-2010 Rafael Gomes <rafaelgomes@techfree.com.br>
#		2010 Elton Pereira <eltonplima@gmail.com>
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# To do : Include code to check when sintax that there isn't in conf

import os
import commands
import stat
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="apache"
        self.long_name="Checks security problems on Apache config file"
        self.type="config"
        self.required_files = ['/etc/httpd/conf/httpd.conf',
                           '/etc/apache2/conf.d/security',
                           '/etc/apache2/apache2.conf']

        options.add_option(
            '--apache_conf',
            action='append',
            dest='apache_conf',
            help='adds a apache configuration file to the list of files to' +
            ' analize'
        )

    def requires(self):
        return self.required_files

    def analyze(self, options):
        """ Analyze Apache config file searching for harmful settings"""

        check_results = self.check_results
        apache_conf_files = self.required_files

        if options.apache_conf:
            for f in options.apache_conf:
                apache_conf_files.append(f)

        apache_conf_file_found = False
        for apache_conf in apache_conf_files:
            if os.path.isfile(apache_conf):
                apache_conf_file_found = True
                fp = None

                try:
                    fp = open(apache_conf, 'r')
                except IOError, (errno, strerror):
                    check_results['info'].append(
                        'Could not open %s: %s' % (apache_conf, strerror)
                    )
                    continue

                lines = [x.strip('\n') for x in fp.readlines()]
                fp.close()

                # Checking if ServerTokens is using harmful conf
                if not 'ServerTokens Minimal' in lines:
                    check_results['ok'].append(
                        'ServerTokens is not using harmful conf'
                    )
                else:
                    check_results['medium'].append(
                        'ServerTokens is using harmful conf (set Minimal)'
                    )

                # Checking if KeepAlive is set to On
                if 'KeepAlive On' in lines:
                    check_results['ok'].append(
                        'KeepAlive is not using harmful conf'
                    )
                else:
                    check_results['medium'].append(
                        'KeepAlive is using harmful conf (set On)'
                    )

                # Checking if ServerSignature is set to On
                if 'ServerSignature Off' in lines:
                    check_results['ok'].append(
                        'ServerSignature is not using harmful conf'
                    )
                else:
                    check_results['medium'].append(
                        'ServerSignature is using harmful conf (set Off)'
                    )

                # Checking if LimitRequestBody is bigger than 0
                if 'LimitRequestBody' in lines:
                    for line in lines:
                        if line.startswith('LimitRequestBody') is True:
                            piece = line.split(' ')
                            if int(piece[1]) == 0:
                                check_results['ok'].append(
                                    'LimitRequestBody is not using harmful' +
                                    ' value (0)'
                                )
                            else:
                                check_results['medium'].append(
                                    'LimitRequestBody is using harmful value' +
                                    ' (0)'
                                )
                else:
                    check_results['ok'].append(
                        'LimitRequestBody is not using harmful value (0)'
                    )

                # Checking if LimitRequestFields is bigger than 0
                if 'LimitRequestFields' in lines:
                    for line in lines:
                        if line.startswith('LimitRequestFields') is True:
                            piece = line.split(' ')
                            if int(piece[1]) == 0:
                                check_results['ok'].append(
                                    'LimitRequestFields is not using harmful' +
                                    ' value (0)'
                                )
                            else:
                                check_results['medium'].append(
                                    'LimitRequestFields is using harmful' +
                                    ' value (0)'
                                )
                else:
                    check_results['ok'].append(
                        'LimitRequestFields is not using harmful value (0)'
                    )

                # Checking if LimitRequestFieldsize is equal 8190
                if 'LimitRequestFieldsize' in lines:
                    for line in lines:
                        if line.startswith('LimitRequestFieldsize') is True:
                            piece = line.split(' ')
                            if int(piece[1]) == 0:
                                check_results['ok'].append(
                                    'LimitRequestFieldsize is using good' +
                                    ' value (8190)'
                                )
                            else:
                                check_results['low'].append(
                                    'LimitRequestFieldsize is not using good' +
                                    ' value (8190)'
                                )
                else:
                    check_results['ok'].append(
                        'LimitRequestFieldsize is using good value (8190)'
                    )

                # Checking if LimitRequestLine is equal 8190
                if 'LimitRequestLine' in lines:
                    for line in lines:
                        if line.startswith('LimitRequestLine') is True:
                            piece = line.split(' ')
                            if int(piece[1]) == 0:
                                check_results['ok'].append(
                                    'LimitRequestLine is using good value' +
                                    ' (8190)'
                                )
                            else:
                                check_results['low'].append(
                                    'LimitRequestLine is not using good' +
                                    ' value (8190)'
                                )
                else:
                    check_results['ok'].append(
                        'LimitRequestLine is using good value (8190)'
                    )

                # Checking Timeout less than 300
                tvalue = 300
                for line in lines:
                    if line.startswith('Timeout') is True:
                        piece = line.split(' ')
                        if int(piece[1]) <= tvalue:
                            check_results['ok'].append(
                                'Timeout is not using harmful value (>=%s)'
                                % (tvalue)
                            )
                        else:
                            check_results['medium'].append(
                                'Timeout is using harmful value (>=%s)'
                                % (tvalue)
                            )

                # Checking if access to Apache manual is enabled
                for line in lines:
                    if line.startswith('Alias /manual/') is True:
                        piece = line.split(' ')
                        if (piece[1]) == '/manual/':
                            check_results['medium'].append(
                                'Access to Apache manual is enabled'
                            )
                        else:
                            check_results['ok'].append(
                                'Access to Apache manual is disabled'
                            )

        # Checking .htpasswd files permission
        mode = "550"
        mode = int(mode, 8)
        locate_status, locate_returns = \
                     commands.getstatusoutput('locate .htpasswd')

        if os.path.exists(locate_returns):
            if locate_status == 0:
                for locate_return in locate_returns.split('\n'):
                    if stat.S_IMODE(os.stat(locate_return).st_mode) == mode:
                        check_results['ok'].append(
                            'The file %s is not using harmful permission (550)'
                            % (locate_return)
                        )
                    else:
                        check_results['medium'].append(\
                            'The file %s is using harmful permission (550)'
                            % (locate_return)
                        )
        else:
            check_results['info'].append(
                'Could not find a .htpasswd file. Please, run updatedb'
            )
        # If there is, closing the apache_config file
        if not apache_conf_file_found:
            check_results['info'].append(
                'Could not find Apache\'s configuration files'
            )

        return check_results


# -*- coding: utf-8 -*-
#
# HnTool rules - remote access
# Copyright (C) 2009-2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="remote_access"
        self.long_name="Checks for services with remote access allowed"
        self.type="config"

    def analyze(self, options):
        check_results = self.check_results
        hosts_allow_file = '/etc/hosts.allow'
        hosts_deny_file  = '/etc/hosts.deny'

        # Checks about the hosts.deny file
        if os.path.isfile(hosts_deny_file):
            fp = open(hosts_deny_file, 'r') #open file
            # getting all the lines from the file
            lines = [x.strip('\n').split(':') for x in fp.readlines()]

            # getting all the lines that are not comments or blanks
            all_access = [x for i, x in enumerate(lines) \
                          if not (x[0].startswith('#') or not x[0] != '')]

            for indexc, line in enumerate(all_access):
                for indexv, col in enumerate(line):
                    all_access[indexc][indexv] = col.strip()

            if all_access:
                for index, service in enumerate(all_access):

                    # if len(service) >= 3 then the file is using 3 parameters
                    if len(service) == 3:
                        #specific service with all access
                        if (service[0] == 'ALL' and \
                            service[1] == 'ALL' and \
                            service[2] == 'DENY'):
                            check_results['ok'].append("By default, services are rejecting connections")

                    # if len(service) == 2 then the file is using 2 parameters
                    elif len(service) == 2:
                        #specific service with all access
                        if (service[0] == 'ALL' and \
                            service[1] == 'ALL'):
                            check_results['ok'].append("By default, services are rejecting connections")
            else:
                check_results['low'].append('Default policy not found')

            #closing file
            fp.close()


        # Checks about the hosts.allow file
        if os.path.isfile(hosts_allow_file):
            fp = open(hosts_allow_file,'r')
            all_access = [x.strip('\n').split(':') for x in fp.readlines()]

            #check access in hosts.allow and remove comments and whitespaces
            all_access = [x for i,x in enumerate(all_access) \
                          if not (x[0].startswith('#') or not (x[0] != ''))]
            for indexc, line in enumerate(all_access):
                for indexv, col in enumerate(line):
                    all_access[indexc][indexv] = col.strip()

            if all_access:
                for index, service in enumerate(all_access):
                    #specific service with all access
                    if service[0] != 'ALL' and service[1] == 'ALL':
                        check_results['medium'].append('Service "' + service[0] + \
                                                       '" accepts remote connections from ALL')
                    #specific service with all access
                    elif service[0] != 'ALL' and service[1] != 'ALL':
                        check_results['medium'].append('Service "' + service[0] + \
                                                       '" accepts remote connections from "' + service[1] + '"')
                    #any service to specific address
                    elif service[0] == 'ALL' and service[1] != 'ALL':
                        check_results['medium'].append('Services are accepting remote access from "' + service[1] + '"')
                    #any service with all access
                    elif service[0] == 'ALL' and service[1] == 'ALL':
                        check_results['high'].append('Services are accepting remote access from ALL')
            #closing file
            fp.close()
        else:
            check_results['ok'].append("There's no service accepting remote connections from ALL")

        return check_results


# -*- coding: utf-8 -*-
#
# HnTool - output module - html
# Copyright (C) 2009-2010 Authors
# Authors:
#   * Hugo Doria <mail at hugodoria dot org>
#   * Aurelio A. Heckert <aurium at gmail dot com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   ( at your option ) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import HnTool.modules
import string

class Format:

    description = "HTML output for a web browser"

    def __init__(self, options):
        pass

    def format_status( self, token ):
        if token == 'ok':
            return '<td class="status-ok">OK</td>'
        elif token == 'low':
            return '<td class="status-low">LOW</td>'
        elif token == 'medium':
            return '<td class="status-medium">MEDIUM</td>'
        elif token == 'high':
            return '<td class="status-high">HIGH</td>'
        elif token == 'info':
            return '<td class="status-info">INFO</td>'

    # Method to show the check results
    def msg_status( self, msg, status ):
        '''
        Method to show the check results
        '''
        return '<tr>' + \
               self.format_status( status ) + \
               '<td>' + msg + '</td>' + \
               '</tr>'

    def output( self, report, conf ):
        self.conf = conf
        # Print all the results, from the 5 types of messages ( ok, low, medium, high and info ).
        # First message is the "ok" one ( m['results'][0] ). The second one is
        # "low" ( m['results'][1] ). The third ( m['results'][2] ) is for "warnings"
        # and the fourth one is "high" ( m['results'][3] ), The last one is for
        # info messages.
        print '''<html>
        <head>
            <title>HnTool - A hardening tool for *nixes - Report</title>

            <style type="text/css">

                body  {
                    font: 12px 'Lucida Grande', sans-serif;
                    color: #666666;
                    text-align: center;
                    margin-right: auto;
                    margin-left: auto;
                    margin-top: 0;
                    border-top: 3px solid black;
                }

                h2 {
                    text-align:  left;
                    font-size:        15px;
                    padding-top:      15px;
                    padding-bottom:   5px;
                    padding-left:     5px;
                    border-bottom: 1px solid #000;
                }

                #wrap {
                    width: 1000px;
                    margin:0 auto;
                    margin-top: 10px;
                    text-align: center;
                }

                #left {
                    width: 700px;
                    float: left;
                }

                #right {
                    margin-top:      15px;
                    margin-left: 720px;
                    border: 1px solid #ddd;
                }

                ul {
                    text-align: left;
                    text-decoration: none;
                }

                table {
                    border: 0;
                    width: 690px;
                }

                td {
                    color: #000;
                    padding: 5px;
                }

                .status-ok {
                    background: lightgreen;
                    text-align: center;
                    font-size: 12px;
                }

                .status-low {
                    background: lightgray;
                    text-align: center;
                    font-size: 12px;
                }

                .status-medium {
                    background: yellow;
                    text-align: center;
                    font-size: 12px;
                }

                .status-high {
                    background: red;
                    text-align: center;
                    font-size: 12px;
                }

                .status-info {
                    background: lightblue;
                    text-align: center;
                    font-size: 12px;
                }
            </style>
        </head>

        <body>

        <div id="wrap">
            <div id="header">
                <h1>HnTool - A hardening tool for *nixes - Report</h1>
            </div>

            <div id="left">
                <table>'''

        statistics = {'ok': 0, 'high': 0, 'medium': 0, 'low': 0, 'info': 0}
        for m in report:
            print '<tr><th colspan="2" align="left"><h2>' + m['title'] + '</h2></th></tr>'
            if m['results']['ok'] != []:
                for result in m['results']['ok']:
                    print self.msg_status( result, 'ok' )
                    statistics['ok'] += 1
            if m['results']['low'] != []:
                for result in m['results']['low']:
                    print self.msg_status( result, 'low' )
                    statistics['low'] += 1
            if m['results']['medium'] != []:
                for result in m['results']['medium']:
                    print self.msg_status( result, 'medium' )
                    statistics['medium'] += 1
            if m['results']['high'] != []:
                for result in m['results']['high']:
                    print self.msg_status( result, 'high' )
                    statistics['high'] += 1
            if m['results']['info'] != []:
                for result in m['results']['info']:
                    print self.msg_status( result, 'info' )
                    statistics['info'] += 1

        print '''
                </table>
            </div> <!-- closing the left div -->

            <div id="right">
                <h3>Statistics</h3>
                <ul>'''

        print '    <li><strong>OK:</strong> ' + str(statistics['ok']) + '</li>'
        print '    <li><strong>HIGH:</strong> ' + str(statistics['high']) + '</li>'
        print '    <li><strong>MEDIUM:</strong> ' + str(statistics['medium']) + '</li>'
        print '    <li><strong>LOW:</strong> ' + str(statistics['low']) + '</li>'
        print '    <li><strong>INFO:</strong> ' + str(statistics['info']) + '</li>'

        print '''
                </ul>
            </div> <!-- closing the right div -->
        </div> <!-- closing the wrap div -->
        </body>
        </html>'''


# -*- coding: utf-8 -*-
#
# HnTool rules - authentication
# Copyright (C) 2009-2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import stat
import HnTool.modules.util
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="authentication"
        self.long_name="Checks users, groups and authentications"
        self.type="config"

    def analyze(self, options):
        check_results = self.check_results
        passwd_file = '/etc/passwd'
        shadow_file = '/etc/shadow'
        logindefs_file = '/etc/login.defs'
        invalid_shell = ['/sbin/nologin', '/bin/false', '/usr/bin/false', \
                         '/var/empty/usr/bin/false', '/usr/sbin/nologin']

        # Checks about the shadow file
        if os.path.isfile(shadow_file):
            # Checking passwd_file permissions
            if oct(os.stat(shadow_file)[stat.ST_MODE] & 0777) == oct(0400):
                check_results['ok'].append('Permissions on shadow file are '+
                                           'correct (400)')
            if oct(os.stat(shadow_file)[stat.ST_MODE] & 0777) == oct(0600):
                check_results['ok'].append('Permissions on shadow file are '+
                                           'correct (600)')
            elif oct(os.stat(shadow_file)[stat.ST_MODE] & 0777) > oct(0600):
                check_results['high'].append('Permissions on shadow file are '+
                                             'greater than 600')

        # Checks about the passwd file
        if os.path.isfile(passwd_file):
            # Checking passwd_file permissions
            if oct(os.stat(passwd_file)[stat.ST_MODE] & 0777) == oct(0644):
                check_results['ok'].append('Permissions on passwd file are '+
                                           'correct (644)')
            elif oct(os.stat(passwd_file)[stat.ST_MODE] & 0777) > oct(0644):
                check_results['high'].append('Permissions on passwd file are '+
                                             'greater than 644')

            # Gets the values of each line of the passwd file
            passwd_fp = open(passwd_file, 'r')
            users = [x.strip('\n').split(':') for x in passwd_fp.readlines()]

            users = [x for i, x in enumerate(users) \
                     if not (x[0].startswith('#') or not (x[0] != ''))]
            for indexc, line in enumerate(users):
                for indexv, col in enumerate(line):
                    users[indexc][indexv] = col.strip()

            if users:

                # will be true if we find users with UID 0
                users_with_uid_zero = False

                # Checking if there's a user (other than root) that has UID 0
                for user in users:
                    if user[0] != 'root' and user[2] == '0':
                        check_results['high'].append('There is a user (not root) ' +
                                                     'with UID 0')
                        users_with_uid_zero = True

                    # Checking if there's a user (other than root)
                    # with a valid shell
                    if user[0] != 'root' and user[-1] not in invalid_shell:
                        check_results['medium'].append('User "' + user[0] + '" may ' +
                                                       'have a harmful shell (' + user[-1] + ')')

                if not users_with_uid_zero:
                    check_results['ok'].append("There aren't users (not root) " +
                                               " with UID 0")

            # closing the passwd file
            passwd_fp.close()

        # Checking permissions on home directories (including /root)
        home_permissions_problems = False
        for dir in os.listdir('/home'):
            if os.path.isdir('/home/' + dir):
                if oct(os.stat('/home/' + dir)[stat.ST_MODE] & 0777) > oct(0100):
                    check_results['medium'].append('Permissions on /home/' + dir +
                                                   ' are greater than 700')
                    home_permissions_problems = True

        # Checking the permissions of the root home directory
        if os.path.exists('/root'):
            if oct(os.stat('/root')[stat.ST_MODE] & 0777) > oct(0750):
                check_results['medium'].append('Permissions on /root dir are '+
                                               'greater than 700')
                home_permissions_problems = True

        if not home_permissions_problems:
            # if we got here, then we didnt found permissions problems
            check_results['ok'].append('Did not found permissions ' +
                                       'problems on home directories')

        # Checks about the login.defs file
        if os.path.isfile(logindefs_file):
            lines = HnTool.modules.util.hntool_conf_parser(logindefs_file)

            # Checking when passwords expires
            if 'PASS_MAX_DAYS' in lines:
                if int(lines['PASS_MAX_DAYS']) > 90:
                    check_results['medium'].append('By default passwords do not ' +
                                                   'expires on 90 days or less')
                else:
                    check_results['ok'].append('By default passwords expires ' +
                                               'on 90 days or less')
            else:
                check_results['high'].append('By default passwords does not expires')

            # Checking the fail delay
            if 'FAIL_DELAY' in lines:
                if int(lines['FAIL_DELAY']) < 3:
                    check_results['medium'].append('Delay between failed login prompts is less than 3s')
                else:
                    check_results['ok'].append('Delay between failed login prompts is more than 3s')
            else:
                check_results['high'].append('Delay between failed login prompts is not defined')

            # Checking pass min days
            if 'PASS_MIN_DAYS' in lines:
                if int(lines['PASS_MIN_DAYS']) < 5:
                    check_results['medium'].append('Min. number of days between password changes is less than 5')
                else:
                    check_results['ok'].append('Min. number of days between password changes is more than 5')
            else:
                check_results['high'].append('Min. number of days between password changes is not defined')

        return check_results


# -*- coding: utf-8 -*-
#
# HnTool rules - system-wide
# Copyright (C) 2009-2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import stat
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="system-wide"
        self.long_name="Checks security problems on system-wide configuration"
        self.type="config"

    def analyze(self, options):
        check_results = self.check_results
        grub_conf_file = ['/boot/grub/menu.lst']
        inittab_file = ['/etc/inittab']
        sysctl_conf_file = ['/etc/sysctl.conf']

        # Checking GRUB configuration file
        for grub_conf in grub_conf_file:
            if os.path.isfile(grub_conf):
                try:
                    fp = open(grub_conf,'r')
                except IOError, (errno, strerror):
                    check_results[4].append('Could not open %s: %s' % (grub_conf, strerror))
                    continue

                grub_conf_lines = [x.strip('\n') for x in fp.readlines()]

                # Getting only the line that starts with password
                password_lines = [x for x in grub_conf_lines if x.startswith('password')]

                # Checking if grub is asking for a password
                # if password_lines size is more than 0 then there's a line
                # starting with 'passwd' on our grub config file
                if len(password_lines) > 0:
                    check_results['ok'].append('GRUB asks for a password')
                else:
                    check_results['low'].append('GRUB does not ask for a password')

                # Closing the grub_conf file
                fp.close()

                # Checking grub_conf permissions
                if oct(os.stat(grub_conf)[stat.ST_MODE] & 0777) == oct(0600):
                    check_results['ok'].append('Permissions on ' + grub_conf +
                                               ' are correct')
                elif oct(os.stat(grub_conf)[stat.ST_MODE] & 0777) > oct(0600):
                    check_results['low'].append('Permissions on ' + grub_conf +
                                                ' are greater than 600')

        # Checking inittab file
        for inittab in inittab_file:
            if os.path.isfile(inittab):
                try:
                    fp = open(inittab, 'r') # open the inittab file
                except IOError, (errno, strerror):
                    check_results[4].append('Could not open %s: %s' % (inittab, strerror))
                    continue

                # Getting the lines from the inititab file
                inittab_lines = [x.strip('\n') for x in fp.readlines()]

                if 'su:S:wait:/sbin/sulogin' in inittab_lines:
                    check_results['ok'].append('Single-User mode requires' +
                                               ' authentication')
                else:
                    check_results['medium'].append('Single-User mode does not' +
                                                   ' requires authentication')

                # Closing the inititab file
                fp.close()

        # Checking sysctl.conf file
        for sysctl in sysctl_conf_file:
            if os.path.isfile(sysctl):
                try:
                    fp = open(sysctl, 'r') # open the sysctl.conf file
                except IOError, (errno, strerror):
                    check_results[4].append('Could not open %s: %s' % (sysctl, strerror))
                    continue

                # Getting the lines from the sysctl configuration file
                # We need to strip all white spaces and \n for each line
                # This way we don't need to worry if the user is using
                # var = 1 (with spaces) or var=1 (w/o spaces) in sysctl.conf
                sysctl_lines = [x.strip('\n').replace(' ', '') for x in fp.readlines()]

                # Checking if core dumps are enabled
                # disabled is good
                if 'fs.suid_dumpable=0' in sysctl_lines:
                    check_results['ok'].append('Core dumps are disabled')
                else:
                    check_results['low'].append('Core dumps are enabled')

                # Checking if exec shield is enabled
                # enabled is good
                if ('kernel.exec-shield=1' and 'kernel.randomize_va_space=1') in sysctl_lines:
                    check_results['ok'].append('ExecShield is enabled')
                else:
                    check_results['low'].append('ExecShield is disabled')

                # Checking if TCP SYN Cookie Protection is enabled
                # enabled is good
                if 'net.ipv4.tcp_syncookies=1' in sysctl_lines:
                    check_results['ok'].append('TCP SYN Cookie Protection is enabled')
                else:
                    check_results['low'].append('TCP SYN Cookie Protection is disabled')

                # Checking if we are ignoring broadcast requests
                # enabled is good
                if 'net.ipv4.icmp_echo_ignore_broadcasts=1' in sysctl_lines:
                    check_results['ok'].append('Ignore broadcast request is enabled')
                else:
                    check_results['low'].append('Ignore broadcast request is disabled')

                # Checking if Ping Reply is disabled
                # disabled is good
                if 'net.ipv4.icmp_echo_ignore_all=1' in sysctl_lines:
                    check_results['ok'].append('Ping reply is disabled')
                else:
                    check_results['low'].append('Ping reply is enabled')

                # Closing the sysctl file
                fp.close()

        return check_results


# -*- coding: utf-8 -*-
#
# HnTool rules - vsftpd
# Copyright (C) 2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="vsftpd"
        self.long_name="Checks security problems on VsFTPd servers"
        self.type="config"
        self.required_files = ['/etc/vsftpd.conf', '/etc/vsftpd/vsftpd.conf']

    def requires(self):
        return self.required_files

    def vsftpdParser(self, pfile):
        '''Method to parse a vsftpd.conf file. Returns a dict with
        all [key, value] of the file.'''

        if os.path.isfile(pfile):
            fp = open(pfile,'r')

            keysValues = {}
            for line in fp.readlines():
                if not line.startswith('#'):
                    line = line.strip().split('=')

                    if len(line) >= 2:
                        keysValues[line[0]] = line[1]

            fp.close()

            return keysValues

    def analyze(self, options):
        check_results = self.check_results
        vsftpd_conf_file = self.required_files

        # getting the lines in a [key. value] format
        for vsftpd_conf in vsftpd_conf_file:
            if os.path.isfile(vsftpd_conf):
                lines = self.vsftpdParser(vsftpd_conf)

        # checking if VsFTPd is running on Standalone method
        if 'listen' in lines:
            if lines['listen'].upper() == 'YES':
                check_results['info'].append('Running on StandAlone')
            else:
                check_results['info'].append('Not running on StandAlone')
        else:
            check_results['info'].append('Running on StandAlone')

        # checking if VsFTPd is using the default port
        if 'listen_port' in lines:
            if int(lines['listen_port']) == 21:
                check_results['info'].append('Using the default port (21)')
            else:
                check_results['info'].append('Not using the default port (21)')
        else:
            check_results['info'].append('Using the default port (21)')

        # checking if chroot is enabled on VsFTPd
        if 'chroot_local_user' in lines:
            if lines['chroot_local_user'].upper() == 'YES':
                check_results['ok'].append('Chrooting local users is enabled')
            else:
                check_results['high'].append('Chrooting local users is disabled')
        else:
            check_results['high'].append('Chrooting local users is disabled')

        # checking if anonymous login is enabled
        if 'anonymous_enable' in lines:
            if lines['anonymous_enable'].upper() == 'YES':
                check_results['info'].append('Anonymous login is allowed')
            else:
                check_results['info'].append('Anonymous login is not allowed')
        else:
            check_results['info'].append('Anonymous login is allowed')

        # checking if ascii_download_enable or ascii_upload_enable is enabled
        if 'ascii_download_enable' in lines or 'ascii_upload_enable' in lines:
            if lines['ascii_download_enable'].upper() == 'YES' or \
            lines['ascii_upload_enable'].upper() == 'YES':
                check_results['high'].append('ASCII mode data transfers is allowed (DoS is possible)')
            else:
                check_results['ok'].append('ASCII mode data transfers is not allowed')
        else:
            check_results['high'].append('ASCII mode data transfers is allowed (DoS is possible)')

        return check_results


#!/usr/bin/env python

from setuptools import setup
from os.path import join
from sys import prefix
from HnTool import __version__

DATAFILES = [
      (join(prefix, 'share', 'man', 'man1'), [join('doc', 'hntool.1')]),
      ('share/doc/hntool-%s' % __version__, ['AUTHORS', 'LICENSE', 'NEWS', 'README', 'TODO'])]

setup(name='HnTool',
      version=str(__version__),
      license='GPL-2',
      description='A hardening tool for *nixes',
      long_description=open('README').read(),
      author='Hugo Doria',
      author_email='hugodoria@gmail.com',
      url='http://code.google.com/p/hntool/',
      packages = ['HnTool', 'HnTool.output', 'HnTool.modules'],
      scripts=['hntool'],
      data_files=DATAFILES)



# -*- coding: utf-8 -*-
#
# HnTool rules - filesystems
# Copyright (C) 2009-2010 Hugo Doria <mail@hugodoria.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import commands
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
	def __init__(self, options):
		MasterRule.__init__(self, options)
		self.short_name="filesystems"
		self.long_name="Checks filesystems for security problems"
		self.type="config"

	def analyze(self, options):
		check_results = self.check_results
		locate_database = {'nix': '/var/lib/mlocate/mlocate.db', \
						   'bsd': '/var/db/locate.database'}
		updatedb_command = {'nix': '/usr/bin/updatedb', \
						    'bsd': '/usr/libexec/locate.updatedb'}

		# Checking if the locate database exists
		if not os.path.isfile(locate_database['nix']) and \
		not os.path.isfile(locate_database['bsd']):
			if os.path.isfile(updatedb_command['nix']):
				check_results['info'].append('%s not found. Please run %s' % \
				(locate_database['nix'], updatedb_command['nix']))
				os_type = 'nix'
			elif os.path.isfile(updatedb_command['bsd']):
				check_results['info'].append('%s not found. Please run %s' % \
				(locate_database['bsd'], updatedb_command['bsd']))
				os_type = 'bsd'
		elif os.path.isfile(locate_database['bsd']):
			check_results['ok'].append('locate.database found.')
			os_type = 'bsd'
		elif os.path.isfile(locate_database['nix']):
			check_results['ok'].append('mlocate.db found.')
			os_type = 'nix'
		# Checking for old files
		datafile = locate_database[os_type]
		files_old = ['/tmp', datafile]

		for files in files_old:
			find_status, find_results = \
			commands.getstatusoutput('find %s -type f -atime +30' % files)
			if find_status != 0:
				check_results['low'].append('Found old file(s) (+30 days) in ' + files)
				if files == datafile:
					check_results['info'].append('Please run %s' % updatedb_command[os_type])
			else:
				check_results['ok'].append('Did not found old file(s) (+30 days) in ' + files)

		return check_results


#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# HnTool - A hardening tool for Linux/BSD
# Copyright (C) 2009-2010 Authors
# Authors:
#   * Hugo Doria <mail at hugodoria dot org>
#   * Aurelio A. Heckert <aurium at gmail dot com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

import sys

import string
from optparse import OptionParser
from optparse import OptionGroup

import gettext
gettext.install('HnTool')

import HnTool.modules.util
import HnTool.modules
import HnTool.output
from HnTool import __version__

class HnToolCore:

    def __init__(self):
        self.rule_modules = {}
        self.output_modules = {}
        self.report = []
        self.options = None

        self.opt_parser = OptionParser(
            usage = _("Usage: %prog [options]"),
            version="%prog " + str(__version__))

        self.output_options = OptionGroup(self.opt_parser, _("Output Options"))
        self.rule_options = OptionGroup(self.opt_parser, _("Rule Options"))

    # Method to list all rules available
    def list_rules(self, *args):
        '''Method to list all rules available'''

        print '-' * 31 + _(' HnTool rule list ') + '-' * 31
        print

        for module in sorted(self.rule_modules):
            print string.ljust(self.rule_modules[module].short_name, 16) +\
                ': ' + self.rule_modules[module].long_name

        sys.exit(0)

    # Method to list all output formats available
    def list_output_formats(self, *args):
        '''Method to list all output formats available'''

        print '-' * 30 + _(' HnTool output list ') + '-' * 30

        for module in self.output_modules:
            print string.ljust(module, 11) + ': ' \
            + self.output_modules[module].description

        sys.exit(0)

    # Loading all rule modules
    def load_modules(self):
        '''Method to load all rule modules'''

        for module in sorted(HnTool.modules.__all__):
            self.rule_modules[module] = \
               __import__('HnTool.modules.' + module, globals(), \
               locals(), [HnTool]).Rule(self.rule_options)

    # Loading all output modules
    def load_output_modules(self):
        '''Method to load all output modules'''

        for module in HnTool.output.__formats__:
            self.output_modules[module] = \
               __import__('HnTool.output.' + module, globals(), \
               locals(), [HnTool]).Format(self.output_options)

    # Parsing all the configuration options
    def config_option_parser(self):
        '''Method to parse all the configuration options'''

        # -l/--list option: list all available rules
        self.opt_parser.add_option("-l", "--list",
                              action="callback",
                              callback=self.list_rules,
                              help=_("returns a list of available rules"))

        # -t/--output_type: select the way that the report will
        # be generate (html or terminal, for now)
        self.opt_parser.add_option("-t", "--output_type",
                              action="store",
                              dest="output_format", default="terminal",
                              help=_("select the output format"))

        # --list_output_type: list all available output formats
        self.opt_parser.add_option("--list_output_type",
                              action="callback",
                              callback=self.list_output_formats,
                              help=_("list the avaliable output formats"))

        # -m/--modules: run specific modules
        self.opt_parser.add_option("-m","--modules",
                              type="string",
                              action="store",
                              dest="modules_list",
                              help=_("run only the tests specified by MODULES_LIST."))

        # -e/--exclude: don't run specific modules
        self.opt_parser.add_option("-e","--exclude",
                              type="string",
                              action="store",
                              dest="exclude_list",
                              help=_("don't run the tests specified by MODULES_LIST."))

        # adding the rules to they respective groups (rules or output)
        self.opt_parser.add_option_group(self.output_options)
        self.opt_parser.add_option_group(self.rule_options)

        self.options, args = self.opt_parser.parse_args(sys.argv[1:])

#TODO: define one error code for each error to allow automatic interactions.

    # Checking if all requirements are met
    def initial_check(self):
        '''Method to check if all HnTool's requirements are met'''

        # yes, only unix for now
        if not HnTool.modules.util.is_unix():
            print >> sys.stderr, \
            _('Error: You must have a Unix(-like) box. (No candy for you)')

            sys.exit(2)

    # Main initialization
    def init_core(self):
        '''Method to run the initial checks and load all modules,
        output modules and configuration options'''

        # status message must not go to stdout to not mess with the output
        #format.
        self.initial_check() # checking HnTool's requirements
        self.load_modules() # loading all the modules
        self.load_output_modules() # loading all the output modules
        self.config_option_parser() # getting all the options

    # This is where we run all the tests
    def run_tests(self):
        '''Method to run all tests available on HnTool'''

        self.init_core() # main initialization

        # checking if we are root. we need to be. oh yeah, baby.
        if not HnTool.modules.util.is_root():
            print >> sys.stderr, _('Error: You must be root to run HnTool')
            print >> sys.stderr, ''
            print >> sys.stderr, self.opt_parser.print_help()

            sys.exit(2)

        # starting the checks/output
        print >> sys.stderr, _('[ Starting HnTool checks... ]')

        #if we used the -e or --exclude option
        if self.options.exclude_list:
            # getting a liste with the difference between the list
            # with all modules and the list passed by the -e option
            modules_list = list(set(HnTool.modules.__all__) -
                                set(self.options.exclude_list.split(',')))

        elif self.options.modules_list: # if we just want to run specific modules
            modules_list = self.options.modules_list.split(',')

        else: # gets all the modules
            modules_list = HnTool.modules.__all__

        # Run all the modules and its checks.
        # The results of each module goes to "report"
        for m in modules_list:

            if m in HnTool.modules.__all__:

                # if the module requires something
                if self.rule_modules[m].requires() != None:

                    # if all the requiremets are met
                    if HnTool.modules.util.requirements_met(self.rule_modules[m].requires()):
                        self.report.append({'title': self.rule_modules[m].long_name, \
                        'results': self.rule_modules[m].analyze(self.options)})

                    else: # if the requirements aren't met
                        tmp_check_results = {'ok': [], 'low': [], 'medium': [],
                                             'high': [], 'info': []}
                        tmp_check_results['info'].append('One or more files required to run ' +
                        'this module could not be found')
                        self.report.append({'title': self.rule_modules[m].long_name, \
                        'results': tmp_check_results})

                # if the module does NOT requires something
                else:
                    self.report.append({'title': self.rule_modules[m].long_name, \
                        'results': self.rule_modules[m].analyze(self.options)})

            else:
                print >> sys.stderr, _('Invalid module ') + m
                sys.exit(2)

        # Give the report to the user
        self.output_modules[self.options.output_format].output(
            self.report,
            self.options
        )

if __name__ == "__main__":
    hn = HnToolCore()
    hn.run_tests()


# -*- coding: utf-8 -*-
#
# HnTool - output module - treminal
# Copyright (C) 2009-2010 Authors
# Authors:
#   * Hugo Doria <mail at hugodoria.org>
#   * Aurelio A. Heckert <aurium at gmail dot com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   ( at your option ) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import HnTool.modules
import string

class Format:

  description = "Human friendly output for terminal"

  def __init__(self, options):
      options.add_option("-n", "--term_nocolors",
                         action="store_false",
                         dest="term_use_colors", default=True,
                         help="does not use colors on terminal output")

  def format_status( self, token ):
      use_colors = self.conf.term_use_colors
      if token == 'ok':
          if use_colors : return '[\033[1;92m   OK   \033[0m]'
          else          : return '[   OK   ]'
      elif token == 'low':
          if use_colors : return '[\033[1;30m  LOW   \033[0m]'
          else          : return '[  LOW   ]'
      elif token == 'medium':
          if use_colors : return '[\033[1;93m MEDIUM \033[0m]'
          else          : return '[ MEDIUM ]'
      elif token == 'high':
          if use_colors : return '[\033[1;91m  HIGH  \033[0m]'
          else          : return '[  HIGH  ]'
      elif token == 'info':
          if use_colors : return '[ \033[37m INFO \033[0m ]'
          else          : return '[  INFO  ]'

  # Method to show the check results
  def msg_status( self, msg, status ):
    '''
    Method to show the check results
    '''
    maxmsg_len = HnTool.modules.util.term_len() - 15
    msg_splited = HnTool.modules.util.split_len( msg, maxmsg_len )
    result = ""
    i = 0
    while i < len(msg_splited) - 1:
      result += "   " + string.ljust( msg_splited[i], maxmsg_len ) + "\n"
      i += 1
    return result + "   " + \
           string.ljust( msg_splited[i], maxmsg_len ) + \
           self.format_status( status )

  def output( self, report, conf ):
    self.conf = conf
    # Print all the results, from the 5 types of messages ( ok, low, medium, high and info ).
    # First message is the "ok" one ( m['results'][0] ). The second one is
    # "low" ( m['results'][1] ). The third ( m['results'][2] ) is for "warnings"
    # and the fourth one is "high" ( m['results'][3] ), The last one is for
    # info messages.

    for m in report:
      if conf.term_use_colors : print '\n \033[96m' + m['title'] + '\033[0m'
      else          : print '\n' + m['title']

      if m['results']['ok'] != []:
        for result in m['results']['ok']:
          print self.msg_status( result, 'ok' )
      if m['results']['low'] != []:
        for result in m['results']['low']:
          print self.msg_status( result, 'low' )
      if m['results']['medium'] != []:
        for result in m['results']['medium']:
          print self.msg_status( result, 'medium' )
      if m['results']['high'] != []:
        for result in m['results']['high']:
          print self.msg_status( result, 'high' )
      if m['results']['info'] != []:
        for result in m['results']['info']:
          print self.msg_status( result, 'info' )


#
# hntool output - __init__
# Copyright (C) 2009-2010 Aurelio A. Heckert <aurium (a) gmail dot com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

__formats__ = ['terminal', 'html']




# -*- coding: utf-8 -*-
#
# HnTool rules - PostgreSQL
# Copyright (C) 2009-2010 Sebastian SWC <mail@sebastianswc.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import shlex
from HnTool.modules.rule import Rule as MasterRule

class Rule(MasterRule):
    def __init__(self, options):
        MasterRule.__init__(self, options)
        self.short_name="postgresql"
        self.long_name="Check security problems on PostgreSQL configuration files"
        self.type="config"
        self.required_files = ['/var/lib/pgsql/data/pg_hba.conf','/var/lib/pgsql/data/postgresql.conf']

    def requires(self):
        return self.required_files

    def analyze(self, options):
        check_results = self.check_results
        pgsql_conf_file = self.required_files

        for pgsql_conf in pgsql_conf_file:
            if os.path.isfile(pgsql_conf):
                try:
                    fp = open(pgsql_conf,'r')
                except IOError, (errno, strerror):
                    check_results['info'].append('Could not open %s: %s' % (pgsql_conf, strerror))
                    continue

                # pg_hba.conf validation
                if 'pg_hba' in pgsql_conf:
                    for line in fp.readlines():
                        if line[0] != '#' and len(line.strip()) > 0:
                            line_conf = shlex.split(line)

                            # check unix sockets authentication
                            if line_conf[0] == 'local':
                                if 'trust' in line:
                                    check_results['medium'].append('Trusted local Unix authentication are allowed')
                                else:
                                    check_results['ok'].append('Trusted local Unix authentication are not allowed')

                            # check tcp/ip host authentication
                            if 'host' in line_conf[0]:
                                if 'trust' in line:
                                    check_results['high'].append('Trusted connection on ' + line_conf[3] + ' are allowed')
                                else:
                                    check_results['ok'].append('Trusted connection on ' + line_conf[3] + ' are not allowed')

                elif 'postgresql' in pgsql_conf:
                    for line in fp.readlines():
                        if len(line.strip()) > 0:
                            line_conf = shlex.split(line)

                            # check the default port
                            if 'port =' in line:
                                if '5432' in line:
                                    check_results['low'].append('Server are running on default port (5432)')
                                else:
                                    check_results['ok'].append('Server are not running at default port (5432)')
                            # check sshl connections
                            if 'ssl =' in line:
                                if 'off' in line:
                                    check_results['low'].append('Server are running without ssl connections support')
                                else:
                                    check_results['ok'].append('Server are running with ssl connections support')
                fp.close()

        return check_results